"""
Type annotations for cloudcontrol service client waiters.

[Open documentation](https://youtype.github.io/types_aiobotocore_docs/types_aiobotocore_cloudcontrol/waiters/)

Usage::

    ```python
    from aiobotocore.session import get_session

    from types_aiobotocore_cloudcontrol.client import CloudControlApiClient
    from types_aiobotocore_cloudcontrol.waiter import (
        ResourceRequestSuccessWaiter,
    )

    session = get_session()
    async with session.create_client("cloudcontrol") as client:
        client: CloudControlApiClient

        resource_request_success_waiter: ResourceRequestSuccessWaiter = client.get_waiter("resource_request_success")
    ```
"""

from aiobotocore.waiter import AIOWaiter

from .type_defs import WaiterConfigTypeDef

__all__ = ("ResourceRequestSuccessWaiter",)

class ResourceRequestSuccessWaiter(AIOWaiter):
    """
    [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/cloudcontrol.html#CloudControlApi.Waiter.ResourceRequestSuccess)
    [Show types-aiobotocore documentation](https://youtype.github.io/types_aiobotocore_docs/types_aiobotocore_cloudcontrol/waiters/#resourcerequestsuccesswaiter)
    """

    async def wait(self, *, RequestToken: str, WaiterConfig: WaiterConfigTypeDef = ...) -> None:
        """
        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/cloudcontrol.html#CloudControlApi.Waiter.ResourceRequestSuccess.wait)
        [Show types-aiobotocore documentation](https://youtype.github.io/types_aiobotocore_docs/types_aiobotocore_cloudcontrol/waiters/#resourcerequestsuccesswaiter)
        """



"""
Type annotations for cloudcontrol service type definitions.

[Open documentation](https://youtype.github.io/types_aiobotocore_docs/types_aiobotocore_cloudcontrol/type_defs/)

Usage::

    ```python
    from types_aiobotocore_cloudcontrol.type_defs import CancelResourceRequestInputRequestTypeDef

    data: CancelResourceRequestInputRequestTypeDef = ...
    ```
"""

import sys
from datetime import datetime
from typing import Dict, List, Sequence

from .literals import HandlerErrorCodeType, OperationStatusType, OperationType

if sys.version_info >= (3, 12):
    from typing import NotRequired
else:
    from typing_extensions import NotRequired
if sys.version_info >= (3, 12):
    from typing import TypedDict
else:
    from typing_extensions import TypedDict

__all__ = (
    "CancelResourceRequestInputRequestTypeDef",
    "ProgressEventTypeDef",
    "ResponseMetadataTypeDef",
    "CreateResourceInputRequestTypeDef",
    "DeleteResourceInputRequestTypeDef",
    "GetResourceInputRequestTypeDef",
    "ResourceDescriptionTypeDef",
    "GetResourceRequestStatusInputRequestTypeDef",
    "WaiterConfigTypeDef",
    "PaginatorConfigTypeDef",
    "ResourceRequestStatusFilterTypeDef",
    "ListResourcesInputRequestTypeDef",
    "UpdateResourceInputRequestTypeDef",
    "CancelResourceRequestOutputTypeDef",
    "CreateResourceOutputTypeDef",
    "DeleteResourceOutputTypeDef",
    "GetResourceRequestStatusOutputTypeDef",
    "ListResourceRequestsOutputTypeDef",
    "UpdateResourceOutputTypeDef",
    "GetResourceOutputTypeDef",
    "ListResourcesOutputTypeDef",
    "GetResourceRequestStatusInputResourceRequestSuccessWaitTypeDef",
    "ListResourcesInputListResourcesPaginateTypeDef",
    "ListResourceRequestsInputListResourceRequestsPaginateTypeDef",
    "ListResourceRequestsInputRequestTypeDef",
)

CancelResourceRequestInputRequestTypeDef = TypedDict(
    "CancelResourceRequestInputRequestTypeDef",
    {
        "RequestToken": str,
    },
)
ProgressEventTypeDef = TypedDict(
    "ProgressEventTypeDef",
    {
        "TypeName": NotRequired[str],
        "Identifier": NotRequired[str],
        "RequestToken": NotRequired[str],
        "Operation": NotRequired[OperationType],
        "OperationStatus": NotRequired[OperationStatusType],
        "EventTime": NotRequired[datetime],
        "ResourceModel": NotRequired[str],
        "StatusMessage": NotRequired[str],
        "ErrorCode": NotRequired[HandlerErrorCodeType],
        "RetryAfter": NotRequired[datetime],
    },
)
ResponseMetadataTypeDef = TypedDict(
    "ResponseMetadataTypeDef",
    {
        "RequestId": str,
        "HostId": str,
        "HTTPStatusCode": int,
        "HTTPHeaders": Dict[str, str],
        "RetryAttempts": int,
    },
)
CreateResourceInputRequestTypeDef = TypedDict(
    "CreateResourceInputRequestTypeDef",
    {
        "TypeName": str,
        "DesiredState": str,
        "TypeVersionId": NotRequired[str],
        "RoleArn": NotRequired[str],
        "ClientToken": NotRequired[str],
    },
)
DeleteResourceInputRequestTypeDef = TypedDict(
    "DeleteResourceInputRequestTypeDef",
    {
        "TypeName": str,
        "Identifier": str,
        "TypeVersionId": NotRequired[str],
        "RoleArn": NotRequired[str],
        "ClientToken": NotRequired[str],
    },
)
GetResourceInputRequestTypeDef = TypedDict(
    "GetResourceInputRequestTypeDef",
    {
        "TypeName": str,
        "Identifier": str,
        "TypeVersionId": NotRequired[str],
        "RoleArn": NotRequired[str],
    },
)
ResourceDescriptionTypeDef = TypedDict(
    "ResourceDescriptionTypeDef",
    {
        "Identifier": NotRequired[str],
        "Properties": NotRequired[str],
    },
)
GetResourceRequestStatusInputRequestTypeDef = TypedDict(
    "GetResourceRequestStatusInputRequestTypeDef",
    {
        "RequestToken": str,
    },
)
WaiterConfigTypeDef = TypedDict(
    "WaiterConfigTypeDef",
    {
        "Delay": NotRequired[int],
        "MaxAttempts": NotRequired[int],
    },
)
PaginatorConfigTypeDef = TypedDict(
    "PaginatorConfigTypeDef",
    {
        "MaxItems": NotRequired[int],
        "PageSize": NotRequired[int],
        "StartingToken": NotRequired[str],
    },
)
ResourceRequestStatusFilterTypeDef = TypedDict(
    "ResourceRequestStatusFilterTypeDef",
    {
        "Operations": NotRequired[Sequence[OperationType]],
        "OperationStatuses": NotRequired[Sequence[OperationStatusType]],
    },
)
ListResourcesInputRequestTypeDef = TypedDict(
    "ListResourcesInputRequestTypeDef",
    {
        "TypeName": str,
        "TypeVersionId": NotRequired[str],
        "RoleArn": NotRequired[str],
        "NextToken": NotRequired[str],
        "MaxResults": NotRequired[int],
        "ResourceModel": NotRequired[str],
    },
)
UpdateResourceInputRequestTypeDef = TypedDict(
    "UpdateResourceInputRequestTypeDef",
    {
        "TypeName": str,
        "Identifier": str,
        "PatchDocument": str,
        "TypeVersionId": NotRequired[str],
        "RoleArn": NotRequired[str],
        "ClientToken": NotRequired[str],
    },
)
CancelResourceRequestOutputTypeDef = TypedDict(
    "CancelResourceRequestOutputTypeDef",
    {
        "ProgressEvent": ProgressEventTypeDef,
        "ResponseMetadata": ResponseMetadataTypeDef,
    },
)
CreateResourceOutputTypeDef = TypedDict(
    "CreateResourceOutputTypeDef",
    {
        "ProgressEvent": ProgressEventTypeDef,
        "ResponseMetadata": ResponseMetadataTypeDef,
    },
)
DeleteResourceOutputTypeDef = TypedDict(
    "DeleteResourceOutputTypeDef",
    {
        "ProgressEvent": ProgressEventTypeDef,
        "ResponseMetadata": ResponseMetadataTypeDef,
    },
)
GetResourceRequestStatusOutputTypeDef = TypedDict(
    "GetResourceRequestStatusOutputTypeDef",
    {
        "ProgressEvent": ProgressEventTypeDef,
        "ResponseMetadata": ResponseMetadataTypeDef,
    },
)
ListResourceRequestsOutputTypeDef = TypedDict(
    "ListResourceRequestsOutputTypeDef",
    {
        "ResourceRequestStatusSummaries": List[ProgressEventTypeDef],
        "NextToken": str,
        "ResponseMetadata": ResponseMetadataTypeDef,
    },
)
UpdateResourceOutputTypeDef = TypedDict(
    "UpdateResourceOutputTypeDef",
    {
        "ProgressEvent": ProgressEventTypeDef,
        "ResponseMetadata": ResponseMetadataTypeDef,
    },
)
GetResourceOutputTypeDef = TypedDict(
    "GetResourceOutputTypeDef",
    {
        "TypeName": str,
        "ResourceDescription": ResourceDescriptionTypeDef,
        "ResponseMetadata": ResponseMetadataTypeDef,
    },
)
ListResourcesOutputTypeDef = TypedDict(
    "ListResourcesOutputTypeDef",
    {
        "TypeName": str,
        "ResourceDescriptions": List[ResourceDescriptionTypeDef],
        "NextToken": str,
        "ResponseMetadata": ResponseMetadataTypeDef,
    },
)
GetResourceRequestStatusInputResourceRequestSuccessWaitTypeDef = TypedDict(
    "GetResourceRequestStatusInputResourceRequestSuccessWaitTypeDef",
    {
        "RequestToken": str,
        "WaiterConfig": NotRequired[WaiterConfigTypeDef],
    },
)
ListResourcesInputListResourcesPaginateTypeDef = TypedDict(
    "ListResourcesInputListResourcesPaginateTypeDef",
    {
        "TypeName": str,
        "TypeVersionId": NotRequired[str],
        "RoleArn": NotRequired[str],
        "ResourceModel": NotRequired[str],
        "PaginationConfig": NotRequired[PaginatorConfigTypeDef],
    },
)
ListResourceRequestsInputListResourceRequestsPaginateTypeDef = TypedDict(
    "ListResourceRequestsInputListResourceRequestsPaginateTypeDef",
    {
        "ResourceRequestStatusFilter": NotRequired[ResourceRequestStatusFilterTypeDef],
        "PaginationConfig": NotRequired[PaginatorConfigTypeDef],
    },
)
ListResourceRequestsInputRequestTypeDef = TypedDict(
    "ListResourceRequestsInputRequestTypeDef",
    {
        "MaxResults": NotRequired[int],
        "NextToken": NotRequired[str],
        "ResourceRequestStatusFilter": NotRequired[ResourceRequestStatusFilterTypeDef],
    },
)



"""
Type annotations for cloudcontrol service client paginators.

[Open documentation](https://youtype.github.io/types_aiobotocore_docs/types_aiobotocore_cloudcontrol/paginators/)

Usage::

    ```python
    from aiobotocore.session import get_session

    from types_aiobotocore_cloudcontrol.client import CloudControlApiClient
    from types_aiobotocore_cloudcontrol.paginator import (
        ListResourceRequestsPaginator,
        ListResourcesPaginator,
    )

    session = get_session()
    with session.create_client("cloudcontrol") as client:
        client: CloudControlApiClient

        list_resource_requests_paginator: ListResourceRequestsPaginator = client.get_paginator("list_resource_requests")
        list_resources_paginator: ListResourcesPaginator = client.get_paginator("list_resources")
    ```
"""

from typing import AsyncIterator, Generic, Iterator, TypeVar

from aiobotocore.paginate import AioPaginator
from botocore.paginate import PageIterator

from .type_defs import (
    ListResourceRequestsOutputTypeDef,
    ListResourcesOutputTypeDef,
    PaginatorConfigTypeDef,
    ResourceRequestStatusFilterTypeDef,
)

__all__ = ("ListResourceRequestsPaginator", "ListResourcesPaginator")

_ItemTypeDef = TypeVar("_ItemTypeDef")


class _PageIterator(Generic[_ItemTypeDef], PageIterator):
    def __iter__(self) -> Iterator[_ItemTypeDef]:
        """
        Proxy method to specify iterator item type.
        """


class ListResourceRequestsPaginator(AioPaginator):
    """
    [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/cloudcontrol.html#CloudControlApi.Paginator.ListResourceRequests)
    [Show types-aiobotocore documentation](https://youtype.github.io/types_aiobotocore_docs/types_aiobotocore_cloudcontrol/paginators/#listresourcerequestspaginator)
    """

    def paginate(
        self,
        *,
        ResourceRequestStatusFilter: ResourceRequestStatusFilterTypeDef = ...,
        PaginationConfig: PaginatorConfigTypeDef = ...,
    ) -> AsyncIterator[ListResourceRequestsOutputTypeDef]:
        """
        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/cloudcontrol.html#CloudControlApi.Paginator.ListResourceRequests.paginate)
        [Show types-aiobotocore documentation](https://youtype.github.io/types_aiobotocore_docs/types_aiobotocore_cloudcontrol/paginators/#listresourcerequestspaginator)
        """


class ListResourcesPaginator(AioPaginator):
    """
    [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/cloudcontrol.html#CloudControlApi.Paginator.ListResources)
    [Show types-aiobotocore documentation](https://youtype.github.io/types_aiobotocore_docs/types_aiobotocore_cloudcontrol/paginators/#listresourcespaginator)
    """

    def paginate(
        self,
        *,
        TypeName: str,
        TypeVersionId: str = ...,
        RoleArn: str = ...,
        ResourceModel: str = ...,
        PaginationConfig: PaginatorConfigTypeDef = ...,
    ) -> AsyncIterator[ListResourcesOutputTypeDef]:
        """
        [Show boto3 documentation](https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/cloudcontrol.html#CloudControlApi.Paginator.ListResources.paginate)
        [Show types-aiobotocore documentation](https://youtype.github.io/types_aiobotocore_docs/types_aiobotocore_cloudcontrol/paginators/#listresourcespaginator)
        """



"""
Main interface for cloudcontrol service.

Usage::

    ```python
    from aiobotocore.session import get_session
    from types_aiobotocore_cloudcontrol import (
        Client,
        CloudControlApiClient,
        ListResourceRequestsPaginator,
        ListResourcesPaginator,
        ResourceRequestSuccessWaiter,
    )

    session = get_session()
    async with session.create_client("cloudcontrol") as client:
        client: CloudControlApiClient
        ...


    resource_request_success_waiter: ResourceRequestSuccessWaiter = client.get_waiter("resource_request_success")

    list_resource_requests_paginator: ListResourceRequestsPaginator = client.get_paginator("list_resource_requests")
    list_resources_paginator: ListResourcesPaginator = client.get_paginator("list_resources")
    ```
"""

from .client import CloudControlApiClient
from .paginator import ListResourceRequestsPaginator, ListResourcesPaginator
from .waiter import ResourceRequestSuccessWaiter

Client = CloudControlApiClient

__all__ = (
    "Client",
    "CloudControlApiClient",
    "ListResourceRequestsPaginator",
    "ListResourcesPaginator",
    "ResourceRequestSuccessWaiter",
)
